# flake8: noqa W605,F821
import copy
import json
import os
import pytest
from idmtools.entities.experiment import Experiment
from idmtools.entities.simulation import Simulation
from idmtools_test import COMMON_INPUT_PATH
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence
from emodpy.emod_task import EMODTask

current_directory = os.path.dirname(os.path.realpath(__file__))
DEFAULT_CONFIG_PATH = os.path.join(COMMON_INPUT_PATH, "files", "config.json")
DEFAULT_CAMPAIGN_JSON = os.path.join(COMMON_INPUT_PATH, "files", "campaign.json")
DEFAULT_DEMOGRAPHICS_JSON = os.path.join(COMMON_INPUT_PATH, "files", "demographics.json")
DEFAULT_ERADICATION_PATH = os.path.join(COMMON_INPUT_PATH, "emod", "Eradication.exe")


@pytest.mark.emod
class TestEMODTask(ITestWithPersistence):

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)

    def test_command(self):
        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication.exe")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation())
        self.assertEqual(r"Assets/Eradication.exe --config config.json --input-path ./Assets;. --dll-path ./Assets",
                         task.command.cmd)

        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation)
        self.assertEqual(r"Assets/Eradication --config config.json --input-path ./Assets;. --dll-path ./Assets",
                         task.command.cmd)

        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication-2.11.custom.exe")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation)
        self.assertEqual(
            r"Assets/Eradication-2.11.custom.exe --config config.json --input-path ./Assets;. --dll-path ./Assets",
            task.command.cmd)

        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "AnotherOne")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation)
        self.assertEqual(r"Assets/AnotherOne --config config.json --input-path ./Assets;. --dll-path ./Assets",
                         task.command.cmd)

    def test_legacy_emod(self):
        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication.exe")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation)
        self.assertEqual(r"Assets/Eradication.exe --config config.json --input-path ./Assets;. --dll-path ./Assets",
                         task.command.cmd)

        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication.exe")
        task = EMODTask(eradication_path=models_dir, legacy_exe=True)
        task.pre_creation(Simulation)
        self.assertEqual(r"Assets/Eradication.exe --config config.json --input-path ./Assets --dll-path ./Assets",
                         task.command.cmd)

    def test_load_files(self):
        task = EMODTask.from_files(
            eradication_path=DEFAULT_ERADICATION_PATH,
            config_path=DEFAULT_CONFIG_PATH,
            campaign_path=DEFAULT_CAMPAIGN_JSON,
            demographics_paths=DEFAULT_DEMOGRAPHICS_JSON
        )

        task.pre_creation(Simulation())
        task.gather_common_assets()

        experiment = Experiment.from_task(copy.deepcopy(task), name=self.case_name)

        # Open all the files for comparison
        with open(DEFAULT_CONFIG_PATH, 'r') as fp:
            config = json.load(fp)["parameters"]
        with open(DEFAULT_CAMPAIGN_JSON, 'r') as fp:
            campaign = json.load(fp)
        with open(DEFAULT_DEMOGRAPHICS_JSON, 'r') as fp:
            demog = json.load(fp)

        task.gather_common_assets()

        self.assertEqual(task.eradication_path, DEFAULT_ERADICATION_PATH)
        self.assertIn(DEFAULT_ERADICATION_PATH, [a.absolute_path for a in task.common_assets.assets])
        self.assertIn(DEFAULT_DEMOGRAPHICS_JSON, [a.absolute_path for a in task.common_assets.assets])
        self.assertDictEqual(demog,
                             task.common_assets.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)).content)

        # check experiment common assets are as expected
        experiment.pre_creation()
        self.assertIn(DEFAULT_ERADICATION_PATH, [a.absolute_path for a in experiment.assets])
        self.assertIn(DEFAULT_DEMOGRAPHICS_JSON, [a.absolute_path for a in experiment.assets])

        sim = experiment.simulations[0]
        # After pre_creation, the demographics filenames will be set on the sim so no more equality
        sim.pre_creation()
        self.assertNotEqual(config["Demographics_Filenames"], sim.task.config["Demographics_Filenames"])
        # Are the demographics set correctly?
        self.assertEqual(sim.task.config["Demographics_Filenames"],
                         [os.path.join("demographics", os.path.basename(DEFAULT_DEMOGRAPHICS_JSON))])
        # The rest should be the same
        del config["Demographics_Filenames"]
        del sim.task.config["Demographics_Filenames"]
        self.set_migrations(config)
        self.assertDictEqual(config, sim.task.config)

        # The demographics coming from the experiment
        self.assertIsNone(sim.task.transient_assets.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)))
        self.assertIsNotNone(sim.task.demographics.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)))

        # No change for campaigns
        self.assertEqual(campaign['Events'], sim.task.campaign.events)
        self.assertEqual(campaign['Campaign_Name'], task.campaign.name)

    def test_load_files_without_demographics(self):
        task = EMODTask.from_files(eradication_path=DEFAULT_ERADICATION_PATH,
                                   config_path=DEFAULT_CONFIG_PATH,
                                   campaign_path=DEFAULT_CAMPAIGN_JSON)

        with open(DEFAULT_CONFIG_PATH, 'r') as fp:
            config = json.load(fp)["parameters"]
        with open(DEFAULT_CAMPAIGN_JSON, 'r') as fp:
            campaign = json.load(fp)

        self.assertDictEqual(config, task.config)
        self.assertEqual(campaign['Campaign_Name'], task.campaign.name)

    def set_migrations(self, dict):
        dict.update({'Enable_Local_Migration': 0})
        dict.update({'Enable_Air_Migration': 0})
        dict.update({'Enable_Family_Migration': 0})
        dict.update({'Enable_Regional_Migration': 0})
        dict.update({'Enable_Sea_Migration': 0})
        return dict
